package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	es2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/utils/es"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/olivere/elastic/v7"
	"github.com/streadway/amqp"
	"time"
)

// EggRecordActiveDataConsume 更新团队活跃次数和签到次数
func EggRecordActiveDataConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>EggRecordActiveDataConsume>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	egg_system_rules.Init(cfg.RedisAddr)
	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			err = handleEggRecordActiveDataConsume(res.Body)
			if err != nil {
				fmt.Println("EggRecordActiveDataConsume_ERR:::::", err.Error())
				utils2.FilePutContents("EggRecordActiveDataConsume_ERR", utils2.SerializeStr(map[string]interface{}{
					"body": res.Body,
					"err":  err.Error(),
				}))
			}
			//_ = res.Reject(false)
			err = res.Ack(true)
			fmt.Println("err ::: ", err)
		} else {
			panic(errors.New("error getting message"))
		}
	}
}

func handleEggRecordActiveDataConsume(msgData []byte) error {
	time.Sleep(time.Duration(100) * time.Millisecond) //休眠100毫秒
	// 1.解析mq中queue的数据结构体
	var msg *md2.EggStructForRecordActiveData
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}

	year, week := time.Now().ISOWeek()
	yearStr := utils2.IntToStr(year)
	weekStr := utils2.IntToStr(week)
	index := es2.GetAppointIndexFromAlias(yearStr, weekStr)

	// 2.获取用户直推上级
	userRelateDb := implement.NewUserRelateDb(db.Db)
	userRelate, err := userRelateDb.GetUserParentUserRelate(msg.Uid)
	if err != nil {
		return err
	}

	// 3.添加团队活跃次数
	if userRelate != nil {
		parentUid := userRelate.ParentUid
		parentEsId := fmt.Sprintf("%d%d-%d", year, week, parentUid)
		// 增加团队活跃次数记录
		script := elastic.NewScript("ctx._source.team_activity_nums += params.inc").Param("inc", 1)
		_, err = es.EsClient.Update().
			Index(index).
			Id(parentEsId).
			Script(script).
			Do(context.Background())
		if err != nil {
			return err
		}
	}

	// 4.更新签到次数
	esId := fmt.Sprintf("%d%d-%d", year, week, msg.Uid)
	script := elastic.NewScript(`
		ctx._source.sign_in_nums += params.signInInc;
	`).Param("signInInc", 1)

	_, err = es.EsClient.Update().
		Index(index).
		Id(esId).
		Script(script).
		Do(context.Background())
	if err != nil {
		return err
	}
	return nil
}
